@@ -5,7 +5,7 @@ use warnings;
 
 use Carp ();
 
-our $VERSION = '0.20';
+our $VERSION = '0.22';
 our $PREFIX  = '__cag_';
 
 require XSLoader;
@@ -13,32 +13,37 @@ XSLoader::load('Class::Accessor::Inherited::XS', $VERSION);
 
 my $REGISTERED_TYPES = {};
 register_types(
-    inherited   => {installer => \&_mk_inherited_accessor},
-    class       => {installer => \&_mk_class_accessor},
-    varclass    => {installer => \&_mk_varclass_accessor},
-    object      => {installer => \&_mk_object_accessor},
-    constructor => {installer => \&_mk_constructor},
+    inherited       => {installer => \&_mk_inherited_accessor,              clone_arg => 1},
+    inherited_ro    => {installer => sub { _mk_inherited_accessor(@_, 1) }, clone_arg => 1},
+    class           => {installer => sub { _mk_class_accessor(@_, 0, 0) },  clone_arg => undef},
+    class_ro        => {installer => sub { _mk_class_accessor(@_, 0, 1) },  clone_arg => undef},
+    varclass        => {installer => sub { _mk_class_accessor(@_, 1, 0) },  clone_arg => undef},
+    varclass_ro     => {installer => sub { _mk_class_accessor(@_, 1, 1) },  clone_arg => undef},
+    object          => {installer => \&_mk_object_accessor,                 clone_arg => 1},
+    object_ro       => {installer => sub { _mk_object_accessor(@_, 1) },    clone_arg => 1},
+    constructor     => {installer => \&_mk_constructor,                     clone_arg => undef},
 );
 
 sub import {
     my $pkg = shift;
     return unless scalar @_;
 
-    my $class = caller;
     my %opts = ref($_[0]) eq 'HASH' ? %{ $_[0] } : @_;
+    my $class = delete $opts{package} // caller;
 
     for my $type (keys %opts) {
         my $accessors = $opts{$type};
         my $installer = _type_installer($type);
+        my $clone_arg = $REGISTERED_TYPES->{$type}{clone_arg};
 
         if (ref($accessors) eq 'HASH') {
             $installer->($class, $_, $accessors->{$_}) for keys %$accessors;
 
         } elsif (ref($accessors) eq 'ARRAY') {
-            $installer->($class, $_, $_) for @$accessors;
+            $installer->($class, $_, $clone_arg && $_) for @$accessors;
 
         } elsif (!ref($accessors)) {
-            $installer->($class, $accessors, $accessors);
+            $installer->($class, $accessors, $clone_arg && $accessors);
 
         } else {
             Carp::confess("Can't understand format for '$type' accessors initializer");
@@ -70,11 +75,14 @@ sub mk_type_accessors {
     my ($class, $type) = (shift, shift);
 
     my $installer = _type_installer($type);
+    my $clone_arg = $REGISTERED_TYPES->{$type}{clone_arg};
+
     for my $entry (@_) {
         if (ref($entry) eq 'ARRAY') {
             $installer->($class, @$entry);
+
         } else {
-            $installer->($class, $entry, $entry);
+            $installer->($class, $entry, $clone_arg && $entry);
         }
     }
 }
@@ -99,6 +107,7 @@ sub register_type {
         };
     }
 
+    $args->{clone_arg} = 1 unless exists $args->{clone_arg};
     $REGISTERED_TYPES->{$type} = $args;
 }
 
@@ -114,27 +123,22 @@ sub _type_installer {
 }
 
 sub _mk_inherited_accessor {
-    my ($class, $name, $field) = @_;
+    my ($class, $name, $field, $is_readonly) = @_;
 
-    install_inherited_accessor("${class}::${name}", $field, $PREFIX.$field);
+    install_inherited_accessor("${class}::${name}", $field, $PREFIX.$field, $is_readonly);
 }
 
 sub _mk_class_accessor {
-    my ($class, $name) = @_;
+    my ($class, $name, $default, $is_varclass, $is_readonly) = @_;
 
-    install_class_accessor("${class}::${name}", 0);
-}
-
-sub _mk_varclass_accessor {
-    my ($class, $name) = @_;
-
-    install_class_accessor("${class}::${name}", 1);
+    install_class_accessor("${class}::${name}", $is_varclass, $is_readonly);
+    ${\$class->$name} = $default if defined $default;
 }
 
 sub _mk_object_accessor {
-    my ($class, $name, $field) = @_;
+    my ($class, $name, $field, $is_readonly) = @_;
 
-    install_object_accessor("${class}::${name}", $field);
+    install_object_accessor("${class}::${name}", $field, $is_readonly);
 }
 
 sub _mk_constructor {
@@ -152,32 +156,31 @@ Class::Accessor::Inherited::XS - Fast XS inherited, object and class accessors
 
 =head1 SYNOPSIS
 
-  #install accessors at compile time
+  # install accessors at compile time
   use Class::Accessor::Inherited::XS 
-      inherited => [qw/foo bar/], # inherited accessors with key names equal to accessor names
-      object    => 'fuz',         # non-inherited object accessor with key name equal to accessor name
-      varclass  => 'boo',         # non-inherited accessor for __PACKAGE__,  aliased with '$__PACKAGE__::boo' variable
-      class     => 'baz',         # non-inherited anonymous accessor for __PACKAGE__
-      constructor => 'new',       # object constructor
+      package      => 'Pkg', # optionally install into another package
+      constructor  => 'new', # object constructor name
+      inherited    => 'foo', # inherited accessor for classes/objects
+      object       => 'foo', # non-inherited simple object accessor
+      varclass     => 'foo', # non-inherited accessor for __PACKAGE__,  aliased with '$__PACKAGE__::foo' variable
+      class        => 'foo', # non-inherited anonymous accessor for __PACKAGE__
   ;
   
-  use Class::Accessor::Inherited::XS { # optional braces
-      inherited => {
-        bar => 'bar_key',
-        foo => 'foo_key',
-      },
-      object    => {fuz => 'fuz_key'},
-      class     => ['baz'],
-      varclass  => ['boo'],
+  use Class::Accessor::Inherited::XS {  # optional braces
+      inherited => {foo => 'foo_key'},  # change package variable/hash key
+      object    => {foo => 'foo_key'},  # change hash key
+      class_ro  => {foo => $default},   # B<class_ro>, B<varclass_ro>, B<class>, B<varclass> set default values instead
+      class     => ['foo', 'bar'],      # provide a list of accessor names
   };
 
-  #or in a Class::Accessor::Grouped-like fashion
+  # or at run time, in a Class::Accessor::Grouped-like fashion
+  # this is unrecommended style and it provides very limited interface
   use parent 'Class::Accessor::Inherited::XS';
 
-  __PACKAGE__->mk_inherited_accessors('foo', ['bar', 'bar_key']);
-  __PACKAGE__->mk_class_accessors('baz');
-  __PACKAGE__->mk_varclass_accessors('boo');
-  __PACKAGE__->mk_object_accessors('fuz');
+  __PACKAGE__->mk_inherited_accessors('foo', [bar => 'bar_key']);
+  __PACKAGE__->mk_class_accessors('foo');
+  __PACKAGE__->mk_varclass_accessors('foo');
+  __PACKAGE__->mk_object_accessors('foo');
 
 =head1 DESCRIPTION
 
@@ -198,6 +201,11 @@ in an anonymous variable.
 
 B<object> accessors provides plain simple hash key access.
 
+There are following read-only accessor types: B<class_ro>, B<varclass_ro>, B<inherited_ro>, B<object_ro>.
+They behave exactly like theirs non-ro counterparts, but will croak when you try to call them with arguments.
+To set values for such accessors, you have to access either corresponding package variable or a hash key. But
+B<ro_class> accessor has none - so you can provide a default value for it using hash initializer syntax.
+
 B<constructor> can create objects either from a list or from a single hashref. Note that if you pass
 a hash reference, it becomes blessed too. If that's not what you want, pass a dereferenced copy.
 
@@ -266,7 +274,8 @@ class_caix       34345065/s          13328%   3044%   2337%             910%
 
 You can register new inherited accessor types with associated read/write callbacks. Unlike
 L<Class::Accessor::Grouped>, only a single callback can be set for a type, without per-class
-B<get_$type>/B<set_$type> lookups.
+B<get_$type>/B<set_$type> lookups. You can omit either B<on_read> or B<on_write> if you don't
+need them to avoid performance losses.
 
 B<on_read> callback receives a single argument - return value from the underlying B<inherited> accessor. It's result
 is the new accessor's return value (and it isn't stored anywhere).
