@@ -12,7 +12,7 @@ CAIXS_payload_attach(pTHX_ CV* cv, AV* keys_av) {
     SvRMAGICAL_off((SV*)cv);
 }
 
-template <AccessorTypes type> static
+template <AccessorType type> static
 shared_keys*
 CAIXS_payload_init(pTHX_ CV* cv) {
     AV* keys_av = newAV();
@@ -24,7 +24,7 @@ CAIXS_payload_init(pTHX_ CV* cv) {
     return (shared_keys*)AvARRAY(keys_av);
 }
 
-template <AccessorTypes type> static
+template <AccessorType type, bool is_readonly> static
 CV*
 CAIXS_install_cv(pTHX_ SV* full_name) {
     STRLEN len;
@@ -36,16 +36,24 @@ CAIXS_install_cv(pTHX_ SV* full_name) {
     }
 #endif
 
-    CV* cv = Perl_newXS_len_flags(aTHX_ full_name_buf, len, &CAIXS_entersub_wrapper<type>, __FILE__, NULL, NULL, SvUTF8(full_name));
+    CV* cv = Perl_newXS_len_flags(aTHX_ full_name_buf, len, (&CAIXS_entersub_wrapper<type, is_readonly>), __FILE__, NULL, NULL, SvUTF8(full_name));
     if (!cv) croak("Can't install XS accessor");
 
     return cv;
 }
 
-template <AccessorTypes type> static
+template <AccessorType type> static
 shared_keys*
-CAIXS_install_accessor(pTHX_ SV* full_name) {
-    CV* cv = CAIXS_install_cv<type>(aTHX_ full_name);
+CAIXS_install_accessor(pTHX_ SV* full_name, bool is_readonly) {
+    CV* cv;
+
+    if (is_readonly) {
+        cv = CAIXS_install_cv<type, true>(aTHX_ full_name);
+
+    } else {
+        cv = CAIXS_install_cv<type, false>(aTHX_ full_name);
+    }
+
     return CAIXS_payload_init<type>(aTHX_ cv);
 }
 
